//----------------------------------------------------------------------------//
//--																		--//
//--    GUIBase.h - GUI Base Class and functions							--//
//--        This header file contains definition of a CAVE menuing			--//
//--    system.  It has been built to be portable and used in other CAVE	--//
//--    application.														--//
//--        It consist of four classes:										--//
//--            tMenu - controls the overall menu.  A variable of this		--//
//--                    type must be defined before anything else is done.	--//
//--            tMenuBase - is a base class for Menu Items.					--//
//--            tMenuItem - contains one Menu Item.							--//
//--            tSubMenu - is both a MenuItem and contains a complete		--//
//--                       menu.											--//
//--																		--//
//----------------------------------------------------------------------------//
//--    written by:    Daniel J. Heath										--//
//--    starting date: January 15, 1996										--//
//----------------------------------------------------------------------------//

#ifndef __GUIBASE__
#define __GUIBASE__

#include <stdio.h>
#include <stdarg.h>
#include <cave_ogl.h>
#include <sharedm.h>

#define GUI_USE_CAVE  0x0001
#define GUI_USE_WORLD 0x0002

#define GUI_SHOWPOINTER			1
#define	GUI_HIDEPOINTER			2
#define GUI_NO_POINTER_PLAIN	3

class tGUIBase;
class tGUIMem;
class tGUIQue;
class tGUIplains;
class tGUIMemResource;

struct tPoint
    {
    float x, y, z;
    };

int		isPointedTowards (const tPoint &p1, const tPoint &dir, const tPoint &p2);
void    calcIntersection(const tPoint &pointInSpace, const tPoint &directionVector,
                             const tPoint &pointInPlane, const tPoint &normal,
                             tPoint &intersection);
void    calcVector(const tPoint &p0, const tPoint &p1, tPoint &v0);
float	scaleInter(const tPoint &a, const tPoint &b, const tPoint &c);
float	calcDistanceSq (const tPoint &a, const tPoint &b);
float   calcMagnitudeUt(const tPoint &v);
float   calcDotProd(const tPoint &v0, const tPoint &v1);
int		isInsideRec(const tPoint &position, const tPoint *box);							 
class tGUIBase
	{
  public:
	static	void		Init		(void);		// To be called from Main Function Before CAVEInit
	static	void		AppInit		(void);		// To be called from CAVEAppInit.
	static	void		Frame		(void);		// To be called from CAVEFrameFunc.
	static	void		Draw		(void);		// To be called from CAVEDrawFunc with original Matrix.
	static  tGUIMemResource	res;				// Used to handle memory

  protected:
						tGUIBase	(int flags);
	virtual				~tGUIBase	();
	virtual tGUIMem*	GUISharedMemInit	(void) {return NULL;}	// Used to Initialized the shared memory
	virtual void		GUIGraphicsInit		(void) {return;}	// Used to Initialize the graphics.
	virtual void		GUIframe			(int) {return;}

			void		*BaseCAVE;
			void		*BaseWorld;
			void		*var;

  private:
	static	int			placePointer (void);
	static	tGUIplains	*calcNearPlain (const tPoint &loc, tPoint dir, tPoint &inter);
	
	static	void		*RootCAVE;
	static	void		*RootWorld;
	static	tGUIBase	*GUIlinks;
	static	void		*svar;
	static  void		*pointer;
	static	void		*pointerTrans;
			tGUIBase	*next;
			int			flags;
	friend tGUIQue;
	friend tGUIplains;
	};

class tGUIMem
	{
  public:
	tGUIMem (void) : draw(0) {}
	int draw;
	};

class tGUIQue
	{
  public:
	virtual	void		doCommand (void) = 0;
  protected:  
			tGUIQue (void);
	virtual	~tGUIQue();
  private:
	tGUIQue	*next;
	friend tGUIBase;
	};

class tGUIplains
	{
  public:
	virtual int		GUIhit		(int) {return 1;}
	
			tPoint		box[5];
			tPoint		normal;
			tPoint		intersection;
	
			void		getXY	(float &x, float &y);
			void		getXY	(tPoint base, tPoint xPoint, tPoint yPoint,
								 float &x, float &y);
			tGUIplains (void);
			~tGUIplains ();
  private:
	tGUIplains	*next;
	tGUIplains	*prev;

	friend tGUIBase;
	};

class tGUIMemResource
	{
  public:
			tGUIMemResource ();
	int		create	(void);
	void	set		(int i, void *value)	{ mem[i] = value; }
	void	*get	(int i)					{ return mem[i];  }
	void	remove	(int i)					{ mem[i] = NULL;  }
  private:  
	void *mem[100];
	int  lastIndex;
	};
	

//======
//======	getXY
inline void tGUIplains::getXY (float &x, float &y)
	{
	x = scaleInter (box[0], box[1], intersection);
	y = scaleInter (box[0], box[3], intersection);
	}
inline void tGUIplains::getXY (tPoint base, tPoint xPoint, tPoint yPoint,
							   float &x, float &y)
	{
	x = scaleInter (base, xPoint, intersection);
	y = scaleInter (base, yPoint, intersection);
	}

#endif
