//---------------------------------------------------------------------------//
//--                                                                       --//
//--    GUI_MenuPriv.h - Open Inventor Menus  (v 1.0a)                       --//
//--        This header file contains definition of a CAVE menuing         --//
//--    system.  It has been built to be portable and used in other CAVE   --//
//--    application.                                                       --//
//--        It consist of four classes:                                    --//
//--            tMenu - controls the overall menu.  A variable of this     --//
//--                    type must be defined before anything else is done. --//
//--            tMenuBase - is a base class for Menu Items.                --//
//--            tMenuItem - contains one Menu Item.                        --//
//--            tSubMenu - is both a MenuItem and contains a complete      --//
//--                       menu.                                           --//
//--                                                                       --//
//---------------------------------------------------------------------------//
//--    written by:    Daniel J. Heath                                     --//
//--    starting date: June 12, 1995                                       --//
//---------------------------------------------------------------------------//

#ifndef OGL_MENUS_H_
#define OGL_MENUS_H_

//======    Include files
//======
#include <cave_ogl.h>
#include <string.h>
#include <math.h>
#include "GUIBase.h"

//======    Menu Return Values
//======
#define	    MENU_NOSELECT	-1
#define	    MENU_SUBMENU    -2

// Type Identification for menus.
#define	    MENUBASE	1
#define	    MENUITEM	2
#define	    SUBMENU		3
#define	    MENULINK	4

//======    Class Definition
//======
class tMenu;
class tMenuShare;
class tMenuPopup;
class tMenuBase;
class tMenuItem;
class tSubMenu;

class tMenuDraw;
class MenuCreate;
class MenuSelect;
class MenuDestroy;


typedef void (*MenuLocCallBack) (float *);
typedef void (*MenuSelectCallBack) (int message, int id, tMenuBase *MB);
//======	Definition of Message
//======
//	MENU_OPEN		subMenuid	subMenu		- Starting the Menu System
//											  A user can update the menu
//											  at this time.
//	MENU_SUB_OPEN	subMenuid	subMenu		- A new Submenu has opened
//											  the user can update the 
//											  menu at this time.
//	MENU_ITEM		id			menuItem	- Selection of a menu item.
//	MENU_SUB_CLOSE	subMenuid	subMenu		- A Submenu is closing
//	MENU_CLOSE		subMenuid	subMenu		- The menu System is closing

#define MENU_OPEN		1
#define	MENU_SUB_OPEN	2
#define MENU_ITEM		3
#define MENU_SUB_CLOSE	4
#define MENU_CLOSE		5
			
//======    Definition of tMenu
//======
class tMenu : private tGUIBase
    {
	friend tMenuPopup;
	friend MenuCreate;
	friend MenuSelect;
	friend MenuDestroy;
  public:
		    tMenu	(void)		    // Constructor
		: tGUIBase (GUI_USE_CAVE) {}

		tGUIMem*	GUISharedMemInit	(void);
		void		GUIGraphicsInit		(void);
		void		GUIframe			(int);

	    void    MakeBase    (tSubMenu *, int = MENU_SUBMENU);
						    // attach menu tree
	    int	    ison	(void);				// return menu statis
	    void    setSubMenu  (void);			// Set the opening submenu to the base menu
	    void    setSubMenu  (tSubMenu *);	// Set the opening submenu to tSubMenu *
	    void    setSubMenu  (int);		    // Set the opening submenu to the submenu with this id
		void	setLocationCallBack (MenuLocCallBack func);
		void	setCallBack			(MenuSelectCallBack func);
	
  private:
	    void    build		(tSubMenu *);	// build a submenu so it can be displayed
	    void    Init		(void);		    // Init global variables
	    void    UnRef		(void);		    // UnRef global variables
		void	callSelFunc	(int message, int id, tMenuBase *MB);
	    tMenuBase* findItem (int, tSubMenu *);  // find the menu item with this id
		int			prevKey;
		
		tMenuShare* var;
		// Graphics Data.
		
		void			*material[4];		// Contains material information
		void			*translate;			// Contains step size for menu item
    };
		
//--------------------------------------------------------------------------------//
//--																			--//
//--	tSBarShare																--//
//--		This contains all shared data for the tSBarBase class.  It is       --//
//--	created during the GUISharedMemInit.									--//
//--																			--//
//--------------------------------------------------------------------------------//
class tMenuShare : public tGUIMem
	{
  public:
	tMemShare (void);

	tSubMenu    *base;			    // contains the base SubMenu
	tSubMenu    *currSM;		    // contains the current SubMenu
	tMenuPopup	*popUp;

	MenuLocCallBack menuLocFunc;	// Contains pointer to a menu
									// location callback.
	MenuSelectCallBack menuSelFunc; // Contains pointer to the menu
									// callback function
	};

class tMenuPopup : public tGUIplains
	{
	friend tMenu;
  public:
	tMenuPopup (tMenu *_menu, int _memres, tSubMenu * _subMenu)
				{
				menu = _menu;
				memres = _memres;
				subMenu = _subMenu;
				curItem = -1;
				prevButton = 1;
				}
	int GUIhit (int);
  private:
	tMenu		*menu;				// Points back to the main structure
	int			memres;				// points to inventor memory.
	tSubMenu	*subMenu;		    // contains the current SubMenus
	int			curItem;			// Itentifies the current select item
	int			prevButton;			// Holds the prev button position.

	void	close (void);
	};

    
//======    Definition of tMenuBase
//======
class tMenuBase
    {
    friend  tMenu;
    friend  tSubMenu;
	friend	tMenuPopup;
	friend	MenuCreate;
	friend	MenuSelect;
	friend	MenuDestroy;
  public:
				tMenuBase (void)
					{
					type = MENUBASE;
					next = NULL;
					}
				tMenuBase*	addItem		(char *name, int id);
				tMenuBase*	addItem		(tSubMenu *SM, char *name, int id = MENU_SUBMENU);
				void		removeItem	(void);
				tMenuBase*	insertItem	(char *name, int id);
				tMenuBase*	insertItem	(tSubMenu *SM, char *name, int id = MENU_SUBMENU);
				tMenuBase*	replaceItem (char *name, int id);
				tMenuBase*	replaceItem	(tSubMenu *SM, char *name, int id = MENU_SUBMENU);
				void		setName		(char *name);
				tSubMenu*	getParent	(void) { return parent; }
	
  protected:
    int			type;		// define type of item
    int         id;			// Identification of menu item
	char		name[255];	// Text String Identifing item
    tMenuBase   *next;		// Next Menu Item
	tMenuBase	*prev;		// Prev Menu Item
	tSubMenu	*parent;	// Parent Menu Structure
    };
    
//======    Definition of tMenuItem
//======
class tMenuItem : public tMenuBase
    {
    friend tMenu;
    friend tSubMenu;
  public:
    tMenuItem (void) { type = MENUITEM; }
    };
    
//======
//======    Definition of tSubMenu
class tSubMenu : private tMenuBase
    {
    friend tMenu;
	friend tMenuBase;
	friend tMenuItem;
	friend tMenuPopup;
	friend MenuCreate;
	friend MenuSelect;
	friend MenuDestroy;
  public:
			tSubMenu	(char *title = "Title");
			~tSubMenu	(void);
    tMenuBase*	addItem		(char *name, int id);                             // add a MenuItem
    tMenuBase*	addItem		(tSubMenu *, char *name, int id = MENU_SUBMENU);  // add SubMenu as a MenuItem
	int			removeItem	(int index);
	tMenuBase*	insertItem	(int index, char *name, int id);
	tMenuBase*	insertItem  (int index, tSubMenu *SM, char *name, int id);
	tMenuBase*	replaceItem (int index, char *name, int id);
	tMenuBase*	replaceItem (int index, tSubMenu *SM, char *name, int id);
	void		setTitle	(char *title);
	tMenuBase*	getItem		(int index);
  private:
	void		subListAdd			(tMenuBase *MB);
	void		subListDelete		(tMenuBase *MB);
	void		subListDeleteAll	(void);
	void		subListInsert		(tMenuBase *after, tMenuBase *MB);
    int			num_item;   // number of items in display;
    tMenuBase   *itemHead;  // list of menu items in it;
	tMenuBase	*itemTail;
    };

struct tMenuDraw
	{
	void			*root;
	int				sizeOfSelect;
	void			*select[25];
	};
/*
class MenuCreate : public tGUIQue
	{
  public:
	MenuCreate	(tMenu *_menu, int _memres,
				 tSubMenu * _subMenu, SbMatrix &_mat)
				{
				menu		= _menu;
				memres		= _memres;
				subMenu		= _subMenu;
				mat			= _mat;
				}
	void	doCommand (void);
  private:
	tMenu *		menu;		//	Points back to main menu class
	int			memres;		//	Points to indivual menu data ?
	tSubMenu *	subMenu;	//	Points to the submenu to create
	SbMatrix	mat;		//	Contains the matrix to position the menu
	};
*/
class MenuSelect : public tGUIQue
	{
  public:
	MenuSelect (int _memres, int _cur, int _index)
		: memres (_memres), cur (_cur), index (_index) {}
	MenuSelect (tMenu *, int _memres, int _cur, int _index)
		: memres (_memres), cur (_cur), index (_index) {}
	void	doCommand (void);
  private:
	int			memres;
	int			cur;
	int			index;
	};
	
class MenuDestroy : public tGUIQue
	{
  public:
	MenuDestroy (tMenu *_menu, tMenuPopup *_box, int _memres)
		{ menu = _menu; box = _box; memres = _memres; };
	void	doCommand (void);  
  private:
	tMenu *		menu;		//	Points back to main menu class
	tMenuPopup *box;		//	Points to the Menu Plane
	int			memres;		//	Points to indivual menu data ?
	};

//======
//======	inline function definitions.
inline void tMenu::setLocationCallBack (MenuLocCallBack func)
	{
	var->menuLocFunc = func;
	}

inline void tMenu::setCallBack (MenuSelectCallBack func)
	{
	var->menuSelFunc = func;
	}

inline int tMenu::ison	(void)
	{return var->draw;}   // return menu statis
inline void tMenu::setSubMenu (void)
	{var->currSM = var->base;} // Set the opening submenu to the base menu
inline void tMenu::callSelFunc (int message, int id, tMenuBase *MB)
	{
	if (var->menuSelFunc)
		(*var->menuSelFunc) (message, id, MB);
	}		
		
inline tMenuBase* tMenuBase::addItem (char *name, int id)
	{
	return insertItem (name, id);
	}
	
inline tMenuBase* tMenuBase::addItem (tSubMenu *SM, char *name, int id)
	{
	return insertItem (SM, name, id);
	}
	    
inline void tMenuBase::setName (char *_name)
	{
	strncpy (name, _name, 255);
	}
	
inline void tSubMenu::setTitle (char *title)
	{
	if (itemHead) itemHead->setName (title);
	}
	

#endif
