//---------------------------------------------------------------------------//
//--                                                                       --//
//--    GUI_SBar.h - GUI Base Class and functions						   --//
//--                                                                       --//
//---------------------------------------------------------------------------//
//--    written by:    Daniel J. Heath                                     --//
//--    starting date: January 15, 1996                                    --//
//---------------------------------------------------------------------------//

#include <GUIBase.h>

//======
//======	Defines for tSBarBase::create
#define	SB_TITLE	0x01
#define SB_HITEXT	0x02
#define SB_CURTEXT	0x04
#define SB_LOWTEXT	0x08
#define SB_TEXT		0x0F

//======
//======	Message arguments returned to CallBack
#define SB_LINEUP		1
#define SB_PAGEUP		2
#define SB_SET_POSITION	3
#define SB_MOV_POSITION 4
#define SB_PAGEDOWN		5
#define SB_LINEDOWN		6

//======
//======	Constants used to define Text Strings
#define SB_TEXT_HI		0
#define SB_TEXT_TITLE	1
#define SB_TEXT_LOW		2
#define	SB_TEXT_CUR		3

//======
//======	Constants used to define colors.
#define SB_COLOR_BOX	0
#define SB_COLOR_TEXT	1	// 2 3
#define SB_COLOR_CYL	4
#define SB_COLOR_TITLE	5	// 6 7
#define SB_COLOR_SLIDER 8
#define SB_COLOR_CUR	9   // 10 11
#define SB_COLOR_NUM	12

//======
//======	Internal Constants
#define GR_BOX_WIDTH	.2
#define GR_CONE_RADIUS	.3
#define GR_CONE_HEIGHT	.5
#define GR_TRAN_LOW		(GR_CONE_HEIGHT * 3 / 2 - GR_BOX_WIDTH / 2)
#define GR_TRAN_HI		(GR_BOX_WIDTH / 2 - GR_CONE_HEIGHT / 2)
#define GR_UNIT_LOW		.4
#define GR_UNIT_HI		(-GR_UNIT_LOW)
#define GR_SLIDER_SIZE	.5

//======
//======	Definitions of Classes
// Main Classes
class tSBarBase;
class tSBarShare;

// Control Indivual Scroll Bars
class tSBar;
class tSBarDraw;

// Graphic Commands
class SBarCreate;
class SBarDestroy;
class SBarLocate;
class SBarText;

//======
//======	Definition of the CallBack
typedef void (*SBarCallback) (tSBar *, int, int, float);

//--------------------------------------------------------------------------------//
//--																			--//
//--	tSBarBase																--//
//--		This is the Base Class that manages all of the Scroll Bars.  At     --//
//--	least one instance of this class must be created before calling         --//
//--	CAVEInit.  It is probably best down with a simple decloration.			--//
//--																			--//
//--------------------------------------------------------------------------------//

class tSBarBase : public tGUIBase
	{
	friend tSBarShare;
	friend tSBar;	
	friend tSBarDraw;
	friend SBarCreate;
	friend SBarDestroy;
	friend SBarLocate;
	friend SBarText;
  public:
	//====== Constructors
	tSBarBase		(void) : tGUIBase (GUI_USE_CAVE) {}
	
	//======	Functions Declorations
			tSBar*	create			(int id, int flags,						// Options
									 tPoint hi, tPoint low, tPoint normal,	// Location
									 char *title, char *hiText,				// Text Strings
									 char *lowText, char *curText,
									 float hiValue, float lowValue,			// Values
									 float curValue, float disValue,
									 float smallStep, float largeStep);
	static	void	destroy			(tSBar *SB);	
			void	setCallBack		(SBarCallback call);	// Sets the Callback function for
															// all scroll bars
  protected:
	virtual tGUIMem*	GUISharedMemInit	(void);
	virtual void		GUIGraphicsInit		(void);
			tSBarShare* var;
  private:
	tSBarDraw	*getSBarDraw (int id);		// Controls the collection of Drawing data for
	tSBarDraw	*barList;					// the indivual scroll bars
	tSBarDraw	*currBar;

	void		*cube;			// Main image of the cube;
	void		*transHi;		// Controls location of Text
	void		*transCur;		//		Contains: SoMaterialBinding
	void		*transLow;		//				  SoTranslation
	void		*transTitle;	// Controls Title Bar
	void		*slider;		// Main Slider
	};
	
//--------------------------------------------------------------------------------//
//--																			--//
//--	tSBarShare																--//
//--		This contains all shared data for the tSBarBase class.  It is       --//
//--	created during the GUISharedMemInit.									--//
//--																			--//
//--------------------------------------------------------------------------------//
class tSBarShare : public tGUIMem
	{
  public:
	SBarCallback	callback;
	float			colors [SB_COLOR_NUM][3];
	void			setColor	(int i, float r, float g, float b);
	};

//--------------------------------------------------------------------------------//
//--																			--//
//--	tSBar																	--//
//--		This class contains and controls one scroll bar.  Creating an       --//
//--	object of this type attomatically links it into the correct link list.  --//
//--	A user should use tSBarBase::create() to generate one of these class.   --//
//--	This will allow for the generations of other classes as will as making  --//
//--	sure that all of the correct fields are updated.						--//
//--	One one copy of this structure exist per scroll bar.					--//
//--																			--//
//--------------------------------------------------------------------------------//

class tSBar : public tGUIplains
	{
	friend tSBarBase;
  public:
	void	destroy			(void);				// Remove this Scroll Bar

	void	setText			(int type, char *text);		// Controls Text Values
	void	setHiValue		(float val);		// Controls data values
	void	setCurValue		(float val);
	void	setCurValue		(float val, float dis);
	void	setCurValue		(float val, char *text);
	void	setLowValue		(float val);

  private:
	int			GUIhit			(int);			//	Handle Scroll Bar hits
	float		FuncReal2unit	(float x);		//	Controls convertion between
	float		FuncUnit2real	(float x);		//		different units of
	float		FuncReal2tran	(float x);		//		messurement.
	//======	Messurement units:
	//======		REAL:	This is repersenitive of the actual value of the field.  It
	//======				is the value that the app program uses when interacting with
	//======				the scroll bars.  hiValue, lowValue and curValue are stored
	//======				in these units.
	//======		UNIT:	This is used to desifer the position of the slider on the unit
	//======				grid given used in a GUIhit function.
	//======		TRAN:	This stands for translation.  It is the value used to translate
	//======				the slider to the correct position.
	//======
	float		unit2realConst;		// Data used by the prev functions.
	float		real2tranConst;
	
	char*		toString (float val);	// Converts a value to a string using curText;
										
	tSBarBase	*SB;		//	Pointer up to the Base Scroll Bar class that created this.
	int			memres;
	int			id;			//	Identification of the Scroll Bar.  This must be unice.
	float		hiValue;	//	Data values for the Scroll Bar.
	float		lowValue;
	float		curValue;
	float		disValue;	//	Value to display, assumed to be equal to curValue;
	float		length;		//	Length of the Scroll Bar.
	float		smallStep;
	float		largeStep;
	char		curText[255];	//	Contains text format string for the current text.

	int			bufferValueChange;	//	Controls the functionality of set{Hi,Cur,Low}Value.
	int			newCurValue;		//		It allows for graphic changes to be postpone until later

	//	This are used in GUIhit to control the scrollbar.
	int			mode;			// What type of movement mode is the scroll bar in?
	int			prevButton;		// Was the button pressed last time I exicuted?
	float		change;
	float		time;			// What time did the changes start.
	};

//--------------------------------------------------------------------------------//
//--																			--//
//--	tSBarDraw																--//
//--		This class contains the data for the open inventor seen graph.      --//
//--	There must be one copy of it per wall.  It is used in strong			--//
//--	conection with tSBar and the SBar Commands.  To accomplish shared		--//
//--	memory pointing at non-shared memory, an identification system was		--//
//--	designed.  Class that wish to use this data, point to tSBarBase and     --//
//--	hold an identifier.  Seeing tSBarBase is non-shared, one copy exist     --//
//--	per wall, and the correct copy is selected based on the what process    --//
//--	is exicuting.  	It then uses the getSBarDraw functions to receive a     --//
//--	pointer to the tSBarDraw that it is interested in.						--//
//--																			--//
//--------------------------------------------------------------------------------//

class tSBarDraw
	{
	friend tSBarBase;
	friend tSBar;
	friend SBarCreate;
	friend SBarDestroy;
	friend SBarLocate;
	friend SBarText;
  private:
	// Constructor / Destructors
	tSBarDraw (tSBarBase *SB);

	tSBarBase		*SBarBase;		//	Information about its Base for proper removal
	int				id;				//	Identification Id.
	tSBarDraw		*next;			//	Link list variables.  Updated by Const/Dest
	tSBarDraw		*prev;

	void			*root;			//	Holds the entire scroll bar
	void			*text[4];		//	Holds Text nodes for modification
	void			*position;		//	Holds location of slider
	};

//--------------------------------------------------------------------------------//
//--																			--//
//--	Command classes:														--//
//--		SBarCreate		- Creates a new scroll bar.							--//
//--		SBarDestroy		- Removes a scroll bar								--//
//--		SBarLocate		- Changes the location of slider and text.			--//
//--		SBarText		- Changes text on the scroll bar.					--//
//--	These should note be application programmer modifable.  There should    --//
//--	be no reason for them to be access directly.							--//
//--																			--//
//--------------------------------------------------------------------------------//

//======
//======	SBarCreate - Creates a new scroll bar and a tSBarDraw class.  It
//======		contains all of the data to do its operation.
/*
class SBarCreate : public tGUIQue
	{
  public:
	void		doCommand (void);	
	tSBarBase	*SB;						//	Contains a pointer back to BASE;
	int			id;							//	Contains an identifier of the scroll bar

	int			flags;						//	Contains option flags
	char		hiText[255],				//	Contains hi text
				curText[255],				//	Contains current text format flat
				lowText[255],				//	Contains low text
				titleText[255];				//	Contains title text
	float		colors [SB_COLOR_NUM][3];	//	Contains definition of Colors
	float		size;						//	Contains the size of the Scroll Bar
	float		pos;						//	Contains position in TRAN units
	float		val;						//	Contains position in REAL units
	SbMatrix	tranMatrix;					//	Contains scroll bar positioning transformation
	};
*/
//======
//======	SBarDestroy - Removes a scroll bar, tSBarDraw and SBar classes assosiated with it.
class SBarDestroy : public tGUIQue
	{
  public:
	SBarDestroy		(tSBarBase *_SB, int _id, tSBar *_SBar)
		{SB = _SB; id = _id ; SBar = _SBar;}
	void		doCommand	(void);
	tSBarBase	*SB;
	int			id;
	tSBar		*SBar;
	};

//======
//======	SBarLocate - Changes the location of the slider. _loc is the TRAN units and
//======		_var is the REAL units.	
class SBarLocate : public tGUIQue
	{
  public:
	SBarLocate	(tSBarBase *_SB, int _id, float _loc, char *text)
		{SB = _SB; id = _id; location = _loc;
		 strcpy (curText, text);}
	void		doCommand	(void);
	tSBarBase	*SB;
	int			id;
	float		location;	//	TRAN units location
	char		curText[255];		//	Display Text
	};

//======
//======	SBarText - Changes a text field
class SBarText : public tGUIQue
	{
  public:
	SBarText (tSBarBase *_SB, int _id, int _textid, char *_text)
		{SB = _SB; id = _id; textid = _textid;
		 strcpy (text, _text);}
	void		doCommand	(void);
	tSBarBase	*SB;
	int			id;
	int			textid;
	char		text[255];
	};

//--------------------------------------------------------------------------------//
//--																			--//
//--	Inline function definitions												--//
//--																			--//
//--------------------------------------------------------------------------------//

inline float tSBar::FuncReal2unit (float x)
	{
	return ((x - lowValue) / unit2realConst + GR_UNIT_LOW);
	}

inline float tSBar::FuncUnit2real (float x)
	{
	return ((x - GR_UNIT_LOW) * unit2realConst + lowValue);
	}

inline float tSBar::FuncReal2tran (float x)
	{
	return ((x - lowValue) * real2tranConst + GR_TRAN_LOW);
	}
	
inline void tSBarBase::setCallBack (SBarCallback call)
	{
	var->callback = call;
	}	

inline void tSBar::destroy (void)
	{
	new SBarDestroy (SB, memres, this);
	}

inline void tSBarBase::destroy (tSBar *SB)
	{
	SB->destroy();	
	}

inline void tSBarShare::setColor (int i, float r, float g, float b)
	{
	colors[i][0] = r;
	colors[i][1] = g;
	colors[i][2] = b;
	}

