/*
 * $Id: hmatrix.h,v 2.0 1995/11/17 16:18:46 ryan Exp ryan $
 *
 * Copyright (c) 1994,1995,  The University of Chicago
 * All rights reserved
 *
 *                       The University of Chicago
 *                        Department of Radiology
 *
 *
 *     Date              : Jan 1995
 *     Author            : Martin Ryan (m-ryan@uchicago.edu)
 *
 *
 * The permission to use, copy, and redistribute all or any part
 * of the included source code is hereby granted, provided that
 * the above copyright notice and the disclaimer appear unmodified
 * in all copies.
 *
 * DISCLAIMER.
 *
 * The software in this package is provided "as is," without any
 * express or implied warranties, including, without limitation,
 * the implied warranties of merchantibility of fitness for a
 * particular purpose. The author and the University of Chicago
 * are not responsible for any direct or indirect damages resulting
 * from use of this code in any form.
 *
 */

#ifndef _HMATRIX_H_
#define _HMATRIX_H_

#include "hvector.h"

class hmatrix { 
protected:
	hvector mx[4];

private:
	friend class hproject;
	friend class phmatrix;

	inline void _zero();
	inline void _clear();
	inline void rotmul(hmatrix&);

	inline float det3x3(float,float,float,
                            float,float,float,
                            float,float,float);

public:
	hmatrix();	// constructor
	hmatrix(hmatrix& hmx);	// constructor
	hmatrix(const float[4][4]);	// constructor
	hmatrix(const hvector&,const hvector&,
		const hvector&,const hvector&);	// constructor
	hmatrix(float,float,float,float,
		float,float,float,float,
		float,float,float,float,
		float,float,float,float);
	hvector& operator[](int);
	float&   operator()(int,int);
	inline float& elem(int i,int j) { return mx[i].v[j]; }
	hvector row(int);	// return a row
	hvector column(int);	// return a column
	inline float*  matrix() { return &mx[0][0]; }

	inline hmatrix operator + (const hmatrix&);
	inline hmatrix operator - (const hmatrix&);
	inline hmatrix operator * (const hmatrix&);
	friend hvector operator * (const hvector&,const hmatrix&);
	friend hvector operator * (const hmatrix&,const hvector&);

	inline hmatrix& operator *= (const float);
	inline hmatrix& operator *= (const hmatrix&);
	friend hvector& operator *= (hvector&,const hmatrix&);

	hmatrix& operator ~ ();

	hmatrix& zero();	// zero out the matrix
	hmatrix& clear();	// set the matrix to Identity

        /*
	 *  affine transformations
	 */
	hmatrix& rotate(float x,float y,float z); // rotate about Z, Y, and X
        // rotate about Z, Y, and X
	hmatrix& rotate(const hvector& hv) {
            return scale(hv.v[0],hv.v[1],hv.v[2]); }
	// rotate about a principle axis
	hmatrix& rotate(float angle,char axis);   
	// rotate about an arbitary axis
	hmatrix& rotate(float angle,hvector& axis); 
	// rotate about polar vector
	hmatrix& rotatePolar(float angle,float theta,float phi); 
	hmatrix& scale(float x,float y,float z);
	hmatrix& scale(const hvector& hv) { 
	    return scale(hv.v[0],hv.v[1],hv.v[2]); }
	hmatrix& translate(float x,float y,float z);
	hmatrix& translate(const hvector& hv) {
	    return translate(hv.v[0],hv.v[1],hv.v[2]); }

	// general matrix ops
	hmatrix& transpose();
	float    det();
	hmatrix  adjoint();
	hmatrix  inverse();
	hmatrix& finv();

	friend ostream& operator << (ostream&,hmatrix&);
	friend istream& operator >> (istream&,hmatrix&);
};


inline hvector hmatrix::row(int rowno)
{
    return mx[rowno];
}
 
inline hvector hmatrix::column(int colno)
{
    hvector c(mx[0][colno],mx[1][colno],mx[2][colno],mx[3][colno]);
    return c;
}

hmatrix  hmatrix::operator + (const hmatrix& hmx2)
{
        hmatrix hmxout;
        for( int i=0; i<4; i++ )
	{
                hmxout.mx[i].v[0] = mx[i].v[0]+hmx2.mx[i].v[0];
                hmxout.mx[i].v[1] = mx[i].v[1]+hmx2.mx[i].v[1];
                hmxout.mx[i].v[2] = mx[i].v[2]+hmx2.mx[i].v[2];
                hmxout.mx[i].v[3] = mx[i].v[3]+hmx2.mx[i].v[3];
	}
        return hmxout;
}

hmatrix  hmatrix::operator - (const hmatrix& hmx2)
{
        hmatrix hmxout;
        for( int i=0; i<4; i++ )
	{
                hmxout.mx[i].v[0] = mx[i].v[0]-hmx2.mx[i].v[0];
                hmxout.mx[i].v[1] = mx[i].v[1]-hmx2.mx[i].v[1];
                hmxout.mx[i].v[2] = mx[i].v[2]-hmx2.mx[i].v[2];
                hmxout.mx[i].v[3] = mx[i].v[3]-hmx2.mx[i].v[3];
	}
        return hmxout;
}

inline hvector	operator * (const hvector& v1,const hmatrix& hmx)
{
	float sum;
	hvector vout;
	for( int i=0; i<4; i++ )
	{
        	sum  = v1.v[0]*hmx.mx[0].v[i];
        	sum += v1.v[1]*hmx.mx[1].v[i];
        	sum += v1.v[2]*hmx.mx[2].v[i];
        	vout.v[i] = sum + v1.v[3]*hmx.mx[3].v[i];
	}
	return vout;
}

inline hvector& operator *= (hvector& v1,const hmatrix& hmx)
{
        float sum;
        hvector vout;
        for( int i=0; i<4; i++ )
        {
                sum  = v1.v[0]*hmx.mx[0].v[i];
                sum += v1.v[1]*hmx.mx[1].v[i];
                sum += v1.v[2]*hmx.mx[2].v[i];
                vout.v[i] = sum + v1.v[3]*hmx.mx[3].v[i];
        }
	v1=vout;
        return v1;
}

inline hvector	operator * (const hmatrix& hmx,const hvector& v1)
{
	float sum;
	hvector vout;
	for( int i=0; i<4; i++ )
	{
	       	sum  = v1.v[0]*hmx.mx[i].v[0];
        	sum += v1.v[1]*hmx.mx[i].v[1];
        	sum += v1.v[2]*hmx.mx[i].v[2];
        	vout.v[i] = sum + v1.v[3]*hmx.mx[i].v[3];
	}
	return vout;
}

hmatrix hmatrix::operator * (const hmatrix& hmx2)
{
	hmatrix hmxout;
	hmxout.zero();
	for( int k=0; k<4; k++ )
		for( int j=0; j<4; j++ )
		{
			hmxout.mx[k].v[0] += mx[k].v[j]*hmx2.mx[j].v[0];
			hmxout.mx[k].v[1] += mx[k].v[j]*hmx2.mx[j].v[1];
			hmxout.mx[k].v[2] += mx[k].v[j]*hmx2.mx[j].v[2];
			hmxout.mx[k].v[3] += mx[k].v[j]*hmx2.mx[j].v[3];
		}
	return hmxout;
}

hmatrix&  hmatrix::operator *= (const float f)
{
        mx[0].v[0] *= f; mx[0].v[1] *= f; mx[0].v[2] *= f; mx[0].v[3] *= f;
        mx[1].v[0] *= f; mx[1].v[1] *= f; mx[1].v[2] *= f; mx[1].v[3] *= f;
        mx[2].v[0] *= f; mx[2].v[1] *= f; mx[2].v[2] *= f; mx[2].v[3] *= f;
        mx[3].v[0] *= f; mx[3].v[1] *= f; mx[3].v[2] *= f; mx[3].v[3] *= f;
        return *this;
}

hmatrix&  hmatrix::operator *= (const hmatrix& hmx2)
{
        hmatrix hmxout;
	hmxout.zero();
        for( int k=0; k<4; k++ )
                for( int j=0; j<4; j++ )
		{
                        hmxout.mx[k].v[0] += mx[k].v[j]*hmx2.mx[j].v[0];
                        hmxout.mx[k].v[1] += mx[k].v[j]*hmx2.mx[j].v[1];
                        hmxout.mx[k].v[2] += mx[k].v[j]*hmx2.mx[j].v[2];
                        hmxout.mx[k].v[3] += mx[k].v[j]*hmx2.mx[j].v[3];
		}
        *this = hmxout;
	return *this;
}

class phmatrix : public phclass {
        const hmatrix* hmp;
	const hvector& row(int i) const { return hmp->mx[i]; }
public:
        phmatrix(const hmatrix&,long =comma);
        phmatrix(const hmatrix&,const char*);
        friend ostream& operator << (ostream&,const phmatrix&);
};

#endif /* _HMATRIX_H_ */
